typedef unsigned short uint8_t;
typedef unsigned int uint16_t;
typedef signed int int16_t;
sbit Chip_Select at PORTB4_bit;
sbit Chip_Select_Direction at DDB4_bit;

int16_t write2bytes(uint16_t command) {
uint8_t higherByte, lowerByte;
int16_t result;

        higherByte = (uint8_t)((command & 0xFF00) >> 8);
        lowerByte = (uint8_t)(command & 0x00FF);

        SPDR = higherByte;  // sending high byte first
        while(!(SPSR & (1<<SPIF)));
        result = SPDR & 0xFF;
        result <<= 8;
        SPDR = lowerByte;  // sending low byte
        while(!(SPSR & (1<<SPIF)));
        result |= SPDR;

        return result;
}//write2bytes

uint8_t SPI_init(void) {

        DDB6_bit = 0;              // Set PB6 pin as input (MISO)
        DDB5_bit = 1;              // Set PB5 pin as output (MOSI)
        Chip_Select = 1;           // Deselect ADC
        Chip_Select_Direction = 1; // Set CS# pin as Output
        SPI1_Init_Advanced(_SPI_MASTER, _SPI_FCY_DIV128, _SPI_CLK_LO_LEADING);

        return 0;
}//SPI_init

uint8_t sendThroughUART(char *text_) {

        UART1_Write_Text(text_);
        UART1_Write(13);
        UART1_Write(10);

        return 0;
}//sendThroughUART

uint8_t startSampling(void) {

        PORTA.F0 = 0;    // CONVSTA = LOW
//        PORTA.F1 = 0;    // CONVSTB = LOW
//        PORTA.F2 = 0;    // CONVSTC = LOW
        Delay_us(1);
        PORTA.F0 = 1;    // CONVSTA = HIGH
//        PORTA.F1 = 1;    // CONVSTB = HIGH
//        PORTA.F2 = 1;    // CONVSTC = HIGH

        return 0;
}//startSampling

int16_t VBs[] = {0, 0, 0, 0, 0, 0};
uint8_t readADC(void) {
uint8_t j;

        Chip_Select = 0; // Select chip
        PORTA.F4 = 0;    // Select RD pin
        for (j = 0; j < 6; j++) // Loop through all six channels getting the high byte, then the low, and moving on to next channel
            VBs[j] = write2bytes(0x0000);
        PORTA.F4 = 1;    // Deselect RD pin
        Chip_Select = 1; // Deselect chip

        return 0;
}//readADC

uint8_t processData(void) {
uint16_t result = 0;
uint8_t j;
char buffer[16];

        for (j = 0; j < 6; j++) {
            result = VBs[j];
            sprintf(buffer, "%d", result);
            sendThroughUART(buffer);
        }//for_j

        UART1_Write_Text("END OF CONVERSION"); UART1_Write(13); UART1_Write(10);

        return 0;
}//processData

void interrupt_ISR () org IVT_ADDR_INT0 {

     SREG_I_bit = 0;        // Disable interrupts
     readADC();
     SREG_I_bit = 1;        // Enable interrupts

     return;
}//interrupt_ISR

uint8_t init(void) {

        DDA0_bit = 1; // Set PA0 as output (CONVSTA)
        DDA1_bit = 1; // Set PA1 as output (CONVSTB)
        DDA2_bit = 1; // Set PA2 as output (CONVSTC)
//        DDA3_bit = 0; // Set PA3 as input (BUSY); it went to INT0
        DDA4_bit = 1; // Set PA4 as output (RD)

        SPI_init();
        UART1_Init(57600);
        Delay_ms(100);
        PORTA.F0 = 1;    // CONVSTA = idle HIGH
//        PORTA.F1 = 1;    // CONVSTB = idle HIGH
//        PORTA.F2 = 1;    // CONVSTC = idle HIGH
        
        EICRA = 0b00000010;   // The falling edge of INT0 generates an interrupt request (BUSY line goes here)
        EIMSK = 0b00000001;   // External interrupt request 0 enable
        SREG_I_bit = 1;       // Enable interrupts

        return 0;
}//init

void main() {

     init();

     while (1) {
        startSampling();
        processData();
        Delay_ms(100);
     }//while

     return;
}//main